precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float progress;
uniform float ratio;
const float start = 0.0;
const float duration = 1.0;
uniform int type;
uniform highp vec2 iResolution;

float mixTime = 0.0;
float iTime;

float rand(float f){
    return fract(sin(mod(dot(f, 12.9898),3.14))*43758.5453);
}

float Less(float x, float value)
{
    return 1.0 - step(value, x);
}

// Will return a value of 1 if the 'x' is >= 'lower' && < 'upper'
float Between(float x, float  lower, float upper)
{
    return step(lower, x) * (1.0 - step(upper, x));
}

//    Will return a value of 1 if 'x' is >= value
float GEqual(float x, float value)
{
    return step(value, x);
}

float wave(float v) {
    float r = 0.0;
    for(float i = 0.0; i< 15.0;i++){
        r += 1.5*sin(v*30.0*rand(i*iTime)*(1.0 + i/3.0)) + 0.2*(rand(v)-0.5);
    }

    return r;
}

vec3 mainCol(vec2 uv) {
    vec3 col = texture2D(inputImageTexture,uv).rgb;
    if (progress > 0.5 - mixTime/2.0/duration && progress < 0.5 + mixTime/2.0/duration) {
        col = mix(texture2D(inputImageTexture,uv).rgb, texture2D(inputImageTexture2,uv).rgb, duration*(progress-0.5 + mixTime/2.0/duration)/mixTime);
    } else if (progress > 0.5) {
        col = texture2D(inputImageTexture2,uv).rgb;
    }
    return col;
}

void main()
{
    iTime = progress * duration + start;
    float jointTime = 0.5 * duration + start;
    vec2 uv = textureCoordinate;

    float v = 4.5;
    if (type == 0 || type == 2) {
        v *= max(iResolution.x, iResolution.y)/iResolution.x;
    } else if (type == 1 || type == 3) {
        v *= max(iResolution.x, iResolution.y)/iResolution.y;
    }
    float len = v*progress + (rand(iTime*3.45)-0.5)*0.5 * step(rand(iTime*6.23), 0.08);

    vec2 uvn = uv;

    if (type == 0 || type == 2) {
        float factor = 2.0*progress;
        if (progress > 0.5){
            factor = 2.0*(1.0-progress);
        }

        // tape wave
        if (type == 0)
        uv.x += factor*wave(uv.y + 0.5) * 0.002;
        else
        uv.x -= factor*wave(uv.y + 0.5) * 0.002;
    }

    vec3 col = mainCol(uvn);
    if (progress>0.0 && progress<1.0){
        float p = ceil(0.5*v);
        float p2 = ceil(1.0*v);

        if (type == 0) {
            uvn.x = mod(uv.x+len, 1.0);
        } else if (type == 1) {
            uvn.y = mod(uv.y+len, 1.0);
        } else if (type == 2) {
            uvn.x = mod(uv.x-len, 1.0);
        } else if (type == 3) {
            uvn.y = mod(uv.y-len, 1.0);
        }

        col = mainCol(uvn);

        if (len > p-1.0 && len < p){
            if (type == 0 || type == 2) {
                if (uv.x + len < p){
                    col = texture2D(inputImageTexture,uvn).rgb;
                } else {
                    col = texture2D(inputImageTexture2,uvn).rgb;
                }
            }else if (type == 1 || type == 3) {
                if (uv.y + len < p){
                    col = texture2D(inputImageTexture,uvn).rgb;
                } else {
                    col = texture2D(inputImageTexture2,uvn).rgb;
                }
            }

        }

        float brightness = 1.0;
        vec2 uvStep;
        uvStep.x = uv.x / (1.0 / iResolution.x);
        uvStep.x = mod(uvStep.x, 3.0);
        uvStep.y = uv.y / (1.0 / iResolution.y);
        uvStep.y = mod(uvStep.y, 3.0);

        vec3 newColour = col;
        float r = 0.7*2.0*progress;
        brightness = 1.0 + 0.25*2.0*progress;
        if (progress > 0.5){
            r = 0.7*2.0*(1.0-progress);
            brightness = 1.0 + 0.25*2.0*(1.0-progress);
        }
        newColour.r = newColour.r * (step(1.0, (Less(uvStep.x, 1.0) + Less(uvStep.y, 1.0)))*r+1.0-r);
        newColour.g = newColour.g * (step(1.0, (Between(uvStep.x, 1.0, 2.0) + Between(uvStep.y, 1.0, 2.0)))*r+1.0-r);
        newColour.b = newColour.b * (step(1.0, (GEqual(uvStep.x, 2.0) + GEqual(uvStep.y, 2.0)))*r+1.0-r);

        col = newColour * brightness;
    }

    gl_FragColor = vec4(col, 1.0);
}