precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float progress;
uniform float ratio;
const float start = 0.0;
const float duration = 1.0;
uniform vec2 iResolution;

#define V vec2(0., 1.)
#define PI 3.14159265
#define validuv(v) (abs(v.x-0.5)<0.5&&abs(v.y-0.5)<0.5)

float mixTime = min(0.1, duration);
float jointTime = start + 0.5*duration;
float iTime;

vec3 mainCol(vec2 uv) {
    vec3 col = texture2D(inputImageTexture, uv).rgb;
    if (progress > 0.5 - mixTime/2.0/duration && progress < 0.5 + mixTime/2.0/duration) {
        col = mix(texture2D(inputImageTexture, uv).rgb, texture2D(inputImageTexture2, uv).rgb, duration*(progress-0.5 + mixTime/2.0/duration)/mixTime);
    } else if (progress > 0.5) {
        col = texture2D(inputImageTexture2, uv).rgb;
    }
    if (!validuv(uv)){
col = vec3(0.0);
}
return col;
}

float rand(float f){
    return fract(sin(mod(dot(f, 12.9898), 3.14))*43758.5453);
}

float v2random(vec2 uv) {
    return rand(uv.x*3.45+uv.y*9.64);
}

float wave(float v) {
    float r = 0.0;
    for (float i = 0.0; i< 15.0;i++){
        r += 1.5*sin(v*40.0*rand(i*iTime)*(1.0 + i/3.0)) + 0.2*(rand(v)-0.5);
    }

    return r;
}

float onOff(float a, float b, float c)
{
    float time = iTime-jointTime+0.17;
    return step(c, sin(time + a*cos(time*b)));
}

vec2 getVideo(vec2 uv)
{
    vec2 look = uv;
    float time = iTime-jointTime+0.17;
    float vShift = 0.4*onOff(2., 3., 0.9)*(sin(time)*sin(time*20.) + (0.5 + 0.1*sin(time*200.)*cos(time)));

    vShift = 0.5*rand(ceil(time/2.0*PI)*start)*onOff(2., 3., 0.9)*(sin(time)*sin(time*20.0) + (0.5 + 0.1*sin(time*200.)*cos(time)));


    look.y = mod(look.y + vShift, 1.);
    return look;
}


void main()
{
    iTime = start + progress*duration;

    vec2 uv = textureCoordinate;
    float time = iTime;

    vec2 uvn = uv;
    vec3 col = vec3(0.0, 0.0, 0.0);

    float factor = 1.4*progress + 0.3;
    if (progress > 0.5){
        factor = 1.4*(1.0-progress) + 0.3;
    }

    if (progress > 0.5 + 10.0/duration) {
        factor = 0.0;
    }

    // tape wave
    uvn.x += factor*wave(uv.y + 0.5) * 0.002;

    // tape crease
    uv.y = ceil(uv.y*iResolution.y/5.0)/iResolution.y;
    float tcPhase = smoothstep(0.7, 0.96, sin(uv.y * 48.0 + (2.0*time + 0.14 * rand(time * 16.74)) * PI * 1.2));
    float tcNoise = smoothstep(0.3, 1.0, rand(uv.y*2.23 + time));

    float tc = tcPhase * tcNoise;
    uvn.x = uvn.x - factor*tc*0.02;

    if (progress>0.0 && progress<1.0 && progress<0.5 + 10.0/duration){
        uvn = getVideo(uvn);
    }

    // fetch
    col = mainCol(uvn);

    // crease noise
    float cn = tcNoise * (0.3 + 0.7 * tcPhase);
    if (0.39 < cn && progress>0.0 && progress<1.0) {
        float n0  = rand(floor((uv.x+iTime+6.9)/(0.03 + 0.02*(rand(uv.y*1.23) - 0.5)))) - 0.1 * fract((uv.x+iTime+6.9)/(0.03 + 0.02*(rand(uv.y*1.23) - 0.5)));
        //float n0  = rand(floor((uv.x+iTime+6.9)/(0.03 + 0.07*rand(uv.y*1.23))))*0.5 + 0.5;

        float rate = 2.0*progress;
        if (progress > 0.5){
            rate = 2.0*(1.0-progress);
        }
        col = mix(col, 1.0 * V.yyy, pow(n0, 10.0) * rate);
    }

    gl_FragColor = vec4(col, 1.0);
}