precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float progress;

float duration = 1.0;
float mixTime = 0.0;

int isY = 0;

float Sinusoidal_easeInOut(in float topValue, in float topTime, in float time) {
    float r = 0.5;
    if (time > r*topTime && time <= topTime){
        return (1.0/r)*topValue*time/topTime - (1.0/r-1.0)*topValue;
    } else if (time > topTime && time <= 1.0-r*(1.0-topTime)){
        return (1.0/r)*(1.0-time)*topValue/(1.0-topTime) - (1.0/r-1.0)*topValue;
    } else {
        return 0.0;
    }
}

float random(in vec3 scale, in float seed) {
    return fract(sin(dot(gl_FragCoord.xyz + seed, scale)) * 43758.5453 + seed);
}

vec2 convertUV(vec2 uv, float progress) {
    uv += 1.0;
    uv *= .5;
    uv = fract(uv);
    uv = abs((uv-0.5) * 2.0);

    if (progress > 0.5) {
        if (isY > 0) {
            uv.y = 1.0 - uv.y;
        } else {
            uv.x = 1.0 - uv.x;
        }
    }
    return uv;
}

vec3 mainCol(vec2 uv, float progress) {
    float sta = 0.5 - mixTime/2.0/duration;
    float end = 0.5 + mixTime/2.0/duration;
    vec3 col = mix(texture2D(inputImageTexture,uv).rgb, texture2D(inputImageTexture2,uv).rgb, smoothstep(sta,end,progress));
    return col;
}

float bezierInterpolation(float t, float a, float b, float c, float d)
{
    return a*(1.0-t)*(1.0-t)*(1.0-t) + 3.0*b*t*(1.0-t)*(1.0-t) + 3.0*c*t*t*(1.0-t) + d*t*t*t;
}

float easeBackByX(float x , vec2 b, vec2 c)
{
    float y = 0.0;
    float y2 = 0.0;
    float interval = 9999.9;
    float nearX = 0.0;
    float nearX2 = 0.0;
    int count = 50;
    for (int i=0; i<count; i++) {
        float t = float(i)* 1.0/(float(count)-1.0);
        float temp = bezierInterpolation(t,0.0,b.x,c.x,1.0);
        if (abs(temp - x) < interval) {
            interval = abs(temp - x);
            nearX2 = nearX;
            y2 = y;
            nearX = temp;
            y = bezierInterpolation(t,0.0,b.y,c.y,1.0);
        }else {
            if (abs(temp - x) < abs(nearX2 - x)) {
                nearX2 = temp;
                y2 = bezierInterpolation(t,0.0,b.y,c.y,1.0);
            }
            break;
        }
    }

    if (abs(nearX2 - nearX) > 0.0) {
        y = y + (x - nearX)*(y2-y)/(nearX2 - nearX);
    }
    return y;
}

void main()
{
    vec2 uv = textureCoordinate;

    float ctrX = 0.0;
    if (uv.x < 0.5 && uv.y < 0.5) {
        ctrX = 1.0;
    } else if (uv.x < 0.5 && uv.y > 0.5) {
        ctrX = 2.0;
    } else if (uv.x > 0.5 && uv.y > 0.5) {
        ctrX = 3.0;
    } else if (uv.x > 0.5 && uv.y < 0.5) {
        ctrX = 4.0;
    }
    ctrX /= 5.0;

    vec2 b = vec2(ctrX, 0.0);
    vec2 c = vec2(0.0, 1.0);
    float newProgress = easeBackByX(progress, b, c);

    float count = 1.0;
    float slope = 0.0;

    float off = (1.0/count)*(1.0-slope)*0.5;

    float strength = Sinusoidal_easeInOut(0.3, 0.5, newProgress);
    float total = 0.0;
    float offset = random(vec3(12.9898, 78.233, 151.7182), 0.0);

    isY = 1;
    if ((uv.x > 0.5 && uv.y>0.5) ||(uv.x < 0.5 && uv.y<0.5)) {
        isY = 0;
    }

    if (isY > 0) {
        uv.y -= (step(fract(floor((uv.x-off)*count - slope*uv.y)/2.0), 0.0) - 0.5)*2.0*newProgress;
    } else {
        uv.x += (step(fract(floor((uv.y-off)*count - slope*uv.x)/2.0), 0.0) - 0.5)*2.0*newProgress;
    }

    vec2 center = vec2(0.5, 0.5);
    float r = 2.0;
    float scale = newProgress*r;
    if (newProgress > 0.5) {
        scale = (1.0 - newProgress)*r;
    }
    uv = (uv - center) * (1.0 - 0.333*scale) + center;

    vec2 distance = vec2(1.0,0.0);
    if (isY > 0) {
        distance = vec2(0.0,1.0);
    } else {
        distance = vec2(1.0,0.0);
    }

    vec3 color = vec3(0.0);
    for (float t = 0.0; t <= 20.0; t++) {
        float percent = (t + offset) / 20.0;
        float weight = 4.0 * (percent - percent * percent);
        vec2 pos = uv + distance * (percent-0.5) * strength;
        color += mainCol(convertUV(pos, newProgress), newProgress) * weight;
        total += weight;
    }
    color /= total;

    gl_FragColor = vec4(color, 1.0);
}