precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float progress;
uniform float ratio;
uniform int type;

float bezierInterpolation(float t, float a, float b, float c, float d)
{
    return a*(1.0-t)*(1.0-t)*(1.0-t) + 3.0*b*t*(1.0-t)*(1.0-t) + 3.0*c*t*t*(1.0-t) + d*t*t*t;
}
float easeBackByX(float x , vec2 b, vec2 c)
{
    float y = 0.0;
    float y2 = 0.0;
    float interval = 9999.9;
    float nearX = 0.0;
    float nearX2 = 0.0;
    int count = 50;
    for (int i=0; i<count; i++) {
        float t = float(i)* 1.0/(float(count)-1.0);
        float temp = bezierInterpolation(t,0.0,b.x,c.x,1.0);
        if (abs(temp - x) < interval) {
            interval = abs(temp - x);
            nearX2 = nearX;
            y2 = y;
            nearX = temp;
            y = bezierInterpolation(t,0.0,b.y,c.y,1.0);
        }else {
            if (abs(temp - x) < abs(nearX2 - x)) {
                nearX2 = temp;
                y2 = bezierInterpolation(t,0.0,b.y,c.y,1.0);
            }
            break;
        }
    }

    if (abs(nearX2-nearX) > 0.0) {
        y = y + (x-nearX)*(y2-y)/(nearX2-nearX);
    }
    return y;
}


float Sinusoidal_easeInOut(in float begin, in float change, in float duration, in float time) {
    if (time <= duration){
        return time*change/duration;
    }else{
        return (duration*change*2.0 - change*time)/duration;
    }
}

float random(in vec3 scale, in float seed) {
    return fract(sin(dot(gl_FragCoord.xyz + seed, scale)) * 43758.5453 + seed);
}

vec3 crossCol(vec2 uv, float rot) {
    float sta = 180.0;
    float end = 180.0;
    vec3 col = mix(texture2D(inputImageTexture,uv).rgb, texture2D(inputImageTexture2,uv).rgb, smoothstep(sta,end,rot));

    return col;
}

vec2 rotate(vec2 uv, float rot) {
    uv -= 0.5;
    mat2 m = mat2(ratio*cos(rot), -ratio*sin(rot), sin(rot), cos(rot));
    uv  = m * uv;
    uv.x /= ratio;

    uv += 0.5;

    uv += 1.0;
    uv *= 0.5;
    uv = fract(uv);
    uv = abs((uv-0.5) * 2.0);

    return uv;
}

void main()
{
    vec2 uv = textureCoordinate;
    float count = 7.0;

    float off = 0.0;
    if (type == 0) {
        off = ceil((uv.y)*count)/(count+1.0);
    } else if (type == 1) {
        off = ceil((uv.x)*count)/(count+1.0);
    } else {
        vec2 tran = vec2(ratio,1.0);
        if (ratio < 1.0) {
            tran = vec2(1.0,1.0/ratio);
        }
        off = ceil((distance((uv-vec2(0.5))*tran+vec2(0.5),vec2(0.5)))*count)/(count+1.0);
    }

    vec2 b = vec2(off, 0.0);
    vec2 c = vec2(0.0, 1.0);
    float newProgress = easeBackByX(progress, b, c);

    float strength = Sinusoidal_easeInOut(0.0, 0.3, 0.5, newProgress);
    float total = 0.0;
    float offset = random(vec3(12.9898, 78.233, 151.7182), 0.0);

    float rot = 360.0 * newProgress;
    rot = clamp(rot, 0.0, 360.0);

    vec3 col = vec3(0.0);
    for (float t = 0.0; t <= 20.0; t++) {
        float percent = (t + offset) / 20.0;
        rot += 10.0*percent * strength * step(0.0, rot);
        vec2 pos = rotate(uv, radians(rot));
        float weight = 4.0 * (percent - percent * percent);
        col += crossCol(pos, rot) * weight;
        total += weight;
    }
    col /= total;

    gl_FragColor = vec4(col, 1.0);
}