precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float progress;

uniform float count;
uniform float tilt;
uniform int type;

float duration = 1.0;
float mixTime = 0.0;

float Sinusoidal_easeInOut(in float topValue, in float topTime, in float time) {
    float r = 0.5;
    if (time > r*topTime && time <= topTime){
        return (1.0/r)*topValue*time/topTime - (1.0/r-1.0)*topValue;
    } else if (time > topTime && time <= 1.0-r*(1.0-topTime)){
        return (1.0/r)*(1.0-time)*topValue/(1.0-topTime) - (1.0/r-1.0)*topValue;
    } else {
        return 0.0;
    }
}

float random(in vec3 scale, in float seed) {
    return fract(sin(dot(gl_FragCoord.xyz + seed, scale)) * 43758.5453 + seed);
}

vec2 convertUV(vec2 uv) {
    uv += 1.0;
    uv *= .5;
    uv = fract(uv);
    uv = abs((uv-0.5) * 2.0);

    if (progress > 0.5) {
        if (type == 0) {
            uv.x = 1.0 - uv.x;
        } else if (type == 1) {
            uv.y = 1.0 - uv.y;
        }else {
            uv.x = 1.0 - uv.x;
            uv.y = 1.0 - uv.y;
        }
    }
    return uv;
}

vec3 mainCol(vec2 uv) {
    float sta = 0.5 - mixTime/2.0/duration;
    float end = 0.5 + mixTime/2.0/duration;
    vec3 col = mix(texture2D(inputImageTexture,uv).rgb, texture2D(inputImageTexture2,uv).rgb, smoothstep(sta,end,progress));
    return col;
}

void main()
{
    vec2 uv = textureCoordinate;
    float off = (1.0/count)*(1.0-tilt)*0.5;
    vec2 pos = uv;
    float factor = 0.167;
    if (type == 0) {
        pos.x -= (step(fract(floor((uv.y+off)*count + tilt*uv.x)/2.0), 0.0) - 0.5)*2.0*progress;
    } else if (type == 1){
        pos.y -= (step(fract(floor((uv.x+off)*count + tilt*uv.y)/2.0), 0.0) - 0.5)*2.0*progress;
    } else if (type == 2){
        pos.x -= (step(fract(floor((uv.y+off)*count + tilt*uv.x)/2.0), 0.0) - 0.5)*2.0*progress;
        pos.y += (step(fract(floor((uv.x+off)*count + tilt*uv.y)/2.0), 0.0) - 0.5)*2.0*progress;
        factor = 0.433;
    }else {
        pos.x -= (step(fract(floor((uv.y+off)*count + tilt*uv.x)/2.0), 0.0) - 0.5)*2.0*progress;
        pos.y += (step(fract(floor((uv.x+off)*count + tilt*uv.y)/2.0), 0.0) - 0.5)*2.0*progress;
        factor = 0.433;
    }

    vec2 center = vec2(0.5, 0.5);
    float r = 2.0;
    float scale = progress*r;
    if (progress > 0.5) {
        scale = (1.0 - progress)*r;
    }
    pos = (pos - center) * (1.0 - factor*scale) + center;

    float strength = Sinusoidal_easeInOut(0.3, 0.5, progress);
    float total = 0.0;
    float offset = random(vec3(12.9898, 78.233, 151.7182), 0.0);

    vec2 distance = vec2(1.0,0.0);
    if (type == 0) {
        distance = vec2(1.0,0.0);
    } else if (type == 1){
        distance = vec2(0.0,1.0);
    } else if (type == 2){
        distance = vec2(1.0,-1.0);
    } else {
        distance = vec2(1.0,1.0);
    }

    vec3 color = vec3(0.0);
    for (float t = 0.0; t <= 20.0; t++) {
        float percent = (t + offset) / 20.0;
        float weight = 4.0 * (percent - percent * percent);
        vec2 new = pos + distance * (percent-0.5) * strength;
        color += mainCol(convertUV(new)) * weight;
        total += weight;
    }
    color /= total;

    gl_FragColor = vec4(color, 1.0);
}