precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float progress;
uniform int type;

float duration = 1.0;
float mixTime = 0.0;

float Sinusoidal_easeInOut(in float topValue, in float topTime, in float time) {
    float r = 0.5;
    if (time > r*topTime && time <= topTime){
        return (1.0/r)*topValue*time/topTime - (1.0/r-1.0)*topValue;
    } else if (time > topTime && time <= 1.0-r*(1.0-topTime)){
        return (1.0/r)*(1.0-time)*topValue/(1.0-topTime) - (1.0/r-1.0)*topValue;
    } else {
        return 0.0;
    }
}

float random(in vec3 scale, in float seed) {
    return fract(sin(dot(gl_FragCoord.xyz + seed, scale)) * 43758.5453 + seed);
}

vec3 mainCol(vec2 uv) {
    float sta = 0.5 - mixTime/2.0/duration;
    float end = 0.5 + mixTime/2.0/duration;
    vec3 col = mix(texture2D(inputImageTexture,uv).rgb, texture2D(inputImageTexture2,uv).rgb, smoothstep(sta,end,progress));
    return col;
}

vec2 convertUV(vec2 uv) {
    vec2 new = uv;

    vec2 center = vec2(0.5, step(uv.x, 0.5));
    if (type == 0) {
        center = vec2(step(uv.y, 0.5), 0.5);
    }
    float r = 2.0;
    float scale = progress*r;
    if (progress > 0.5) {
        scale = (1.0 - progress)*r;
    }

    new = (uv - center) * (1.0 - 0.091*scale) + center;

    if(progress > 0.5){
        scale = (1.0 - progress - 0.0)*r;
        scale = max(0.0, scale);

        new = uv;
        new = (new - center) * (1.0 - 0.07*scale) + center;


        if (type == 0) {
            new.x += (step(new.y, 0.5) - 0.5)*2.0*((progress-0.5)*1.0 -0.5);
        } else {
            new.y += (step(new.x, 0.5) - 0.5)*2.0*((progress-0.5)*1.0 -0.5);
        }

        new += 1.0;
        new *= .5;
        new = fract(new);
        new = abs((new-0.5) * 2.0);
    }
    return new;
}

void main()
{
    vec2 uv = textureCoordinate;
    vec2 pos = uv;
    vec3 color = vec3(0.0);
    float strength = Sinusoidal_easeInOut(0.3, 0.5, progress);
    float total = 0.0;
    float offset = random(vec3(12.9898, 78.233, 151.7182), 0.0);
    vec2 direct = vec2(0.0,1.0);
    if (type == 0) {
        direct = vec2(1.0, 0.0);
    }
    for (float t = 0.0; t <= 20.0; t++) {
        float percent = (t + offset) / 20.0;
        float weight = 4.0 * (percent - percent * percent);
        vec2 new = pos + direct * (percent-0.5) * strength;
        color += mainCol(convertUV(new)) * weight;
        total += weight;
    }
    color /= total;

    gl_FragColor = vec4(color, 1.0);
}