precision highp float;
varying highp vec2 textureCoordinate;
uniform highp float progress;
uniform highp int type;
uniform float ratio;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

const float PI = 3.141592653589793;

float Sinusoidal_easeInOut(in float begin, in float change, in float duration, in float time) {
    if (time <= duration){
        return time*change/duration;
    }else{
        return (duration*change*2.0 - change*time)/duration;
    }
}

float random(in vec3 scale, in float seed) {
    return fract(sin(dot(gl_FragCoord.xyz + seed, scale)) * 43758.5453 + seed);
}

vec3 crossFade(in vec2 uv, in float pr) {
    uv = vec2(abs(1.0 - abs(uv.x - 1.0)),abs(1.0 - abs(uv.y - 1.0)));
    return mix(texture2D(inputImageTexture, uv).rgb, texture2D(inputImageTexture2, uv).rgb, step(0.5, pr));
}

void main()
{
    float strength = Sinusoidal_easeInOut(0.0, 0.3, 0.5, progress);
    vec3 color = vec3(0.0);
    float total = 0.0;
    float offset = random(vec3(12.9898, 78.233, 151.7182), 0.0);

    vec2 coord = textureCoordinate;
    vec2 co = vec2(100.0,100.0/ratio);
    vec2 fco = vec2(co.x*coord.x,co.y*coord.y);
    float sin_factor;
    float cos_factor;

    if (type == 0) {
        if(progress <= 0.5) {
            sin_factor = sin(-90.0*progress*2.0/180.0*PI);
            cos_factor = cos(-90.0*progress*2.0/180.0*PI);
        }else {
            sin_factor = sin(-90.0*(-1.0 + (progress-0.5)*2.0)/180.0*PI);
            cos_factor = cos(-90.0*(-1.0 + (progress-0.5)*2.0)/180.0*PI);
        }
    }else {
        if(progress <= 0.5) {
            sin_factor = sin(90.0*progress*2.0/180.0*PI);
            cos_factor = cos(90.0*progress*2.0/180.0*PI);
        }else {
            sin_factor = sin(90.0*(-1.0 + (progress-0.5)*2.0)/180.0*PI);
            cos_factor = cos(90.0*(-1.0 + (progress-0.5)*2.0)/180.0*PI);
        }
    }

    coord = vec2((fco.x - co.x/2.0)*cos_factor - (fco.y - co.y/2.0)*sin_factor + co.x/2.0, (fco.x - co.x/2.0)*sin_factor + (fco.y - co.y/2.0)*cos_factor + co.y/2.0);

    for (float t = 0.0; t <= 20.0; t++) {
        float percent = (t + offset) / 20.0;
        float weight = 4.0 * (percent - percent * percent);
        float sin = sin(90.0* percent * strength*2.0/180.0*PI);
        float cos = cos(90.0* percent * strength*2.0/180.0*PI);
        vec2 nextCoord = vec2((coord.x - co.x/2.0)*cos - (coord.y - co.y/2.0)*sin + co.x/2.0, (coord.x - co.x/2.0)*sin + (coord.y - co.y/2.0)*cos + co.y/2.0);
        nextCoord = vec2(nextCoord.x/co.x,nextCoord.y/co.y);
        color += crossFade(nextCoord, progress) * weight;
        total += weight;
    }

    gl_FragColor = vec4(color / total, 1.0);
}