precision highp float;
varying highp vec2 textureCoordinate;
uniform highp float progress;
uniform highp int direction;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

const float PI = 3.141592653589793;

float Sinusoidal_easeInOut(in float begin, in float change, in float duration, in float time) {
    if (time <= duration){
        return time*change/duration;
    }else{
        return (duration*change*2.0 - change*time)/duration;
    }
}

float random(in vec3 scale, in float seed) {
    return fract(sin(dot(gl_FragCoord.xyz + seed, scale)) * 43758.5453 + seed);
}

vec3 crossFade(in vec2 uv, in float pr) {
    uv = vec2(abs(1.0 - abs(uv.x - 1.0)),abs(1.0 - abs(uv.y - 1.0)));
    return mix(texture2D(inputImageTexture, uv).rgb, texture2D(inputImageTexture2, uv).rgb, step(0.5, pr));
}

void main()
{
    float strength = Sinusoidal_easeInOut(0.0, 0.3, 0.5, progress);
    vec3 color = vec3(0.0);
    float total = 0.0;
    float offset = random(vec3(12.9898, 78.233, 151.7182), 0.0);
    vec2 co = vec2(.0,.0);
    vec2 distance = vec2(.0,.0);

    if (direction == 0) {
        if (progress <= 0.5) {
            co = textureCoordinate+vec2(1.0,0.0)*progress*2.0;
        }else {
            co = textureCoordinate-vec2(1.0,0.0)+vec2(1.0,0.0)*(progress-0.5)*2.0;
        }
        distance = vec2(1.0,0.0);
    }else if (direction == 1) {
        if (progress <= 0.5) {
            co = textureCoordinate-vec2(1.0,0.0)*progress*2.0;
        }else {
            co = textureCoordinate+vec2(1.0,0.0)-vec2(1.0,0.0)*(progress-0.5)*2.0;
        }
        distance = vec2(1.0,0.0);
    }else if (direction == 2) {
        if (progress <= 0.5) {
            co = textureCoordinate+vec2(0.0,1.0)*progress*2.0;
        }else {
            co = textureCoordinate-vec2(0.0,1.0)+vec2(0.0,1.0)*(progress-0.5)*2.0;
        }
        distance = vec2(0.0,1.0);
    }else if (direction == 3){
        if (progress <= 0.5) {
            co = textureCoordinate-vec2(0.0,1.0)*progress*2.0;
        }else {
            co = textureCoordinate+vec2(0.0,1.0)-vec2(0.0,1.0)*(progress-0.5)*2.0;
        }
        distance = vec2(0.0,1.0);
    }else if (direction == 4) {
        if (progress <= 0.5) {
            co = textureCoordinate+vec2(1.0,1.0)*progress*2.0;
        }else {
            co = textureCoordinate-vec2(1.0,1.0)+vec2(1.0,1.0)*(progress-0.5)*2.0;
        }
        distance = vec2(1.0,1.0);
    }else if (direction == 5) {
        if (progress <= 0.5) {
            co = textureCoordinate+vec2(1.0,-1.0)*progress*2.0;
        }else {
            co = textureCoordinate-vec2(1.0,-1.0)+vec2(1.0,-1.0)*(progress-0.5)*2.0;
        }
        distance = vec2(1.0,-1.0);
    }else if (direction == 6) {
        if (progress <= 0.5) {
            co = textureCoordinate+vec2(-1.0,1.0)*progress*2.0;
        }else {
            co = textureCoordinate-vec2(-1.0,1.0)+vec2(-1.0,1.0)*(progress-0.5)*2.0;
        }
        distance = vec2(-1.0,1.0);
    }else {
        if (progress <= 0.5) {
            co = textureCoordinate-vec2(1.0,1.0)*progress*2.0;
        }else {
            co = textureCoordinate+vec2(1.0,1.0)-vec2(1.0,1.0)*(progress-0.5)*2.0;
        }
        distance = vec2(-1.0,-1.0);
    }

    for (float t = 0.0; t <= 20.0; t++) {
        float percent = (t + offset) / 20.0;
        float weight = 4.0 * (percent - percent * percent);
        color += crossFade(co + distance * (percent-0.5) * strength, progress) * weight;
        total += weight;
    }

    gl_FragColor = vec4(color / total, 1.0);

}