precision highp float;
varying highp vec2 textureCoordinate;
uniform highp float progress;
uniform highp int type;
uniform highp float ratio;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

const float PI = 3.141592653589793;

float Sinusoidal_easeInOut(in float begin, in float change, in float duration, in float time) {
    if (time <= duration){
        return time*change/duration;
    }else{
        return (duration*change*2.0 - change*time)/duration;
    }
}

float random(in vec3 scale, in float seed) {
    return fract(sin(dot(gl_FragCoord.xyz + seed, scale)) * 43758.5453 + seed);
}

vec3 crossFade(in vec2 uv, in float pr) {
    uv = vec2(abs(1.0 - abs(uv.x - 1.0)),abs(1.0 - abs(uv.y - 1.0)));
    return mix(texture2D(inputImageTexture, uv).rgb, texture2D(inputImageTexture2, uv).rgb, step(0.5, pr));
}

void main()
{
    float strength = Sinusoidal_easeInOut(0.0, 0.3, 0.5, progress);
    vec3 color = vec3(0.0);
    float total = 0.0;
    float offset = random(vec3(12.9898, 78.233, 151.7182), 0.0);
    vec2 coord = textureCoordinate;
    vec2 co = vec2(100.0,100.0/ratio);
    vec2 fco = vec2(co.x*coord.x,co.y*coord.y);

    if (type == 0) {
        float zoomOut = 0.5;
        float zoomIn = 1.5;

        if(progress <= 0.5) {
            if(fco.x <= co.x/2.0) {
                fco.x=co.x/2.0 - (co.x/2.0 - fco.x)/(1.0 - (1.0 - zoomOut)*progress*2.0);
            }else{
                fco.x=co.x/2.0 + (fco.x - co.x/2.0)/(1.0 - (1.0 - zoomOut)*progress*2.0);
            }
            if(fco.y <= co.y/2.0) {
                fco.y=co.y/2.0 - (co.y/2.0 - fco.y)/(1.0 - (1.0 - zoomOut)*progress*2.0);
            }else{
                fco.y=co.y/2.0 + (fco.y - co.y/2.0)/(1.0 - (1.0 - zoomOut)*progress*2.0);
            }
            coord = fco;
            coord = vec2(coord.x/co.x,coord.y/co.y);
        }else {
            if(fco.x <= co.x/2.0) {
                fco.x=co.x/2.0 - (co.x/2.0 - fco.x)/(zoomIn - (zoomIn - 1.0)*(progress-0.5)/0.5);
            }else{
                fco.x=co.x/2.0 + (fco.x - co.x/2.0)/(zoomIn - (zoomIn - 1.0)*(progress-0.5)/0.5);
            }
            if(fco.y <= co.y/2.0) {
                fco.y=co.y/2.0 - (co.y/2.0 - fco.y)/(zoomIn - (zoomIn - 1.0)*(progress-0.5)/0.5);
            }else{
                fco.y=co.y/2.0 + (fco.y - co.y/2.0)/(zoomIn - (zoomIn - 1.0)*(progress-0.5)/0.5);
            }
            coord = fco;
            coord = vec2(coord.x/co.x,coord.y/co.y);
        }
    }else{
        float zoomOut = 0.5;
        float zoomIn = 1.3;

        if(progress <= 0.5) {
            if(fco.x <= co.x/2.0) {
                fco.x=co.x/2.0 - (co.x/2.0 - fco.x)/(1.0 + (zoomIn - 1.0)*progress*2.0);
            }else{
                fco.x=co.x/2.0 + (fco.x - co.x/2.0)/(1.0 + (zoomIn - 1.0)*progress*2.0);
            }
            if(fco.y <= co.y/2.0) {
                fco.y=co.y/2.0 - (co.y/2.0 - fco.y)/(1.0 + (zoomIn - 1.0)*progress*2.0);
            }else{
                fco.y=co.y/2.0 + (fco.y - co.y/2.0)/(1.0 + (zoomIn - 1.0)*progress*2.0);
            }
            coord = fco;
            coord = vec2(coord.x/co.x,coord.y/co.y);
        }else {
            if(fco.x <= co.x/2.0) {
                fco.x=co.x/2.0 - (co.x/2.0 - fco.x)/(zoomOut + (1.0 - zoomOut)*(progress-0.5)/0.5);
            }else{
                fco.x=co.x/2.0 + (fco.x - co.x/2.0)/(zoomOut + (1.0 - zoomOut)*(progress-0.5)/0.5);
            }
            if(fco.y <= co.y/2.0) {
                fco.y=co.y/2.0 - (co.y/2.0 - fco.y)/(zoomOut + (1.0 - zoomOut)*(progress-0.5)/0.5);
            }else{
                fco.y=co.y/2.0 + (fco.y - co.y/2.0)/(zoomOut + (1.0 - zoomOut)*(progress-0.5)/0.5);
            }
            coord = fco;
            coord = vec2(coord.x/co.x,coord.y/co.y);
        }
    }

    vec2 toCenter = vec2(0.5,0.5) - coord;

    for (float t = 0.0; t <= 20.0; t++) {
        float percent = (t + offset) / 20.0;
        float weight = 4.0 * (percent - percent * percent);
        color += crossFade(coord + toCenter * percent * strength, progress) * weight;
        total += weight;
    }

    gl_FragColor = vec4(color / total, 1.0);
}