precision highp float;

varying vec2 outTexCoord;

uniform sampler2D texture;
uniform sampler2D lutTexture;
uniform sampler2D pipixiaTexture;   // 用opencv识别出贴纸出的贴纸描边
uniform float intensity;
uniform int multiplyColorAfterSamplingEnabled;
uniform vec4 multiplyColorAfterSampling;

uniform vec4 outlineColor;
uniform highp float outlineThreshold;

void main() {
    vec4 textureColor = texture2D(texture, outTexCoord);
    if (textureColor.a > 0.5) {
        if (multiplyColorAfterSamplingEnabled == 1) {
            textureColor *= multiplyColorAfterSampling;
        }
        float blueColor = textureColor.b * 63.0;
        vec2 quad1;
        quad1.y = floor(floor(blueColor) / 8.0);
        quad1.x = floor(blueColor) - (quad1.y * 8.0);
        vec2 quad2;
        quad2.y = floor(ceil(blueColor) / 8.0);
        quad2.x = ceil(blueColor) - (quad2.y * 8.0);
        vec2 texPos1;
        texPos1.x = (quad1.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
        texPos1.y = (quad1.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);
        vec2 texPos2;
        texPos2.x = (quad2.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
        texPos2.y = (quad2.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);
        vec4 newColor1 = texture2D(lutTexture, texPos1);
        vec4 newColor2 = texture2D(lutTexture, texPos2);
        vec4 newColor = mix(newColor1, newColor2, fract(blueColor));
        gl_FragColor = mix(textureColor, vec4(newColor.rgb, textureColor.w), intensity);
    } else {
        vec4 aux = texture2D(pipixiaTexture, outTexCoord);
        vec4 base = outlineColor * (clamp(outlineThreshold - aux.r, 0.0, 0.04) / 0.04);
        gl_FragColor = base + textureColor - base * textureColor.a;
    }
}